/* Simple interface module for standardized calls of Motif widgets */
/* simplifies the parameter lists. All functions have fully
/* type-checked prototypes */
/* Lutz Prechelt, 1995/05/17 */

#include <Xm/FileSB.h>
#include <Xm/Label.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Text.h>
#include <X11/Shell.h>

#include "stdmotif.h"

/******************** internals ********************/

static char *selected_file;


/*************** Motif extensions ********************************/

/* type-checkable resource name constants */
XmIntResourceName XmCcolumns =      { XmNcolumns },
                  XmCnumColumns =   { XmNnumColumns },
                  XmCorientation =  { XmNorientation },
                  XmCpacking =      { XmNpacking },
                  XmCrows =         { XmNrows },
                  XmCwidth =        { XmNwidth };
XmFuncResourceName XmCcancelCallback =   { XmNcancelCallback },
                   XmChelpCallback =     { XmNhelpCallback },
                   XmCokCallback =       { XmNokCallback },
                   XmCactivateCallback = { XmNactivateCallback };
XmStringResourceName XmCvalue =          { XmNvalue };
XmWidgetResourceName XmCworkWindow =     { XmNworkWindow };



/* ----- create a popup dialog window widget with an OK button */
Widget XmCreateErrorDialogOK (Widget parent, String resourcename,
                              XmString message)
{
  Arg args[5];
  int n = 0;
  Widget dialog;
  XtSetArg (args[n], XmNmessageString, message); n++;
  /* remove Cancel and Help buttons??? see page 151 */
  dialog = XmCreateErrorDialog (parent, resourcename, args, n);
  return (dialog);
}


/* ----- create file selector dialog box */
Widget XmCreateFileSelectorDialog (Widget w, String name)
{
  return (XmCreateFileSelectionDialog (w, name, NULL, 0));
}


/* ----- create a label widget with given label text */
Widget XmCreateLabelWidget (String resourcename, Widget parent, 
                            XmString text)
{
  Widget label = XtVaCreateManagedWidget (resourcename, xmLabelWidgetClass,
                                          parent, XmNlabelString, text, NULL);
  return (label);
}


/* ----- create a bare MainWindow widget */
Widget XmCreateMainWindowWidget (String resourcename, Widget parent)
{
  Widget w = XtVaCreateManagedWidget (resourcename, xmMainWindowWidgetClass,
                                      parent, NULL);
  return (w);
}


/* ----- create a pulldown menu with exactly three entries */
Widget XmCreatePulldownMenu3 (String resourcename, Widget menubar, 
                              int menunumber,
                              XmString entry1, char key1,
                              XmString entry2, char key2,
                              XmString entry3, char key3,
                              void (*callback) (Widget, XtPointer, XtPointer))
{
  Widget menu = XmVaCreateSimplePulldownMenu (menubar, resourcename, 
                                              menunumber, callback,
                           XmVaPUSHBUTTON, entry1, key1, NULL, NULL,
                           XmVaPUSHBUTTON, entry2, key2, NULL, NULL,
                           XmVaPUSHBUTTON, entry3, key3, NULL, NULL,
                           NULL);
  return (menu);
}


/* ----- create a PushButton widget with a given label */
Widget XmCreatePushButtonL (String resourcename, Widget parent,
                            XmString label)
{
  return (XtVaCreateManagedWidget (resourcename, xmPushButtonWidgetClass,
                                   parent, XmNlabelString, label, NULL));
}


/* ----- create a horizontal or vertical RowColumn manager */
Widget XmCreateRowColumnManager (String resourcename, Widget parent)
{
  return (XtVaCreateManagedWidget 
             (resourcename, xmRowColumnWidgetClass, parent, NULL));
}


/* ----- create a horizontal or vertical RowColumn manager */
Widget XmCreateRowColumnManagerOCP (String resourcename, Widget parent,
                                    int orientation, int numColumns,
                                    Boolean equalize)
{
  return (XtVaCreateManagedWidget 
             (resourcename, xmRowColumnWidgetClass, parent,
              XmNorientation, orientation,
              XmNpacking, equalize ? XmPACK_COLUMN : XmPACK_TIGHT,
              XmNnumColumns, numColumns, NULL));
}


/* ----- create a scrolled text widget without any resources set */
Widget XmCreateScrolledTextWidget (Widget parent, String resourcename)
{
  Widget st;
  Arg args[2];
  int n = 0;
  XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
  st = XmCreateScrolledText (parent, resourcename, args, n);
  XtManageChild (st);
  return (st);
}

/* ----- create a stand alone scrolled text widget of lines x columns */
Widget XmCreateScrolledTextWindow (String resourcename, Widget parent,
                                   int nrOfLines, int nrOfColumns)
{
  Widget shell = XtCreatePopupShell (resourcename, topLevelShellWidgetClass,
                                     parent, NULL, 0),
         scrolltext = XmCreateScrolledTextWidget (shell, "scrldtxt");
  XtSetIntValue (scrolltext, XmCrows, nrOfLines);
  XtSetIntValue (scrolltext, XmCcolumns, nrOfColumns);
  XtManageChild (shell);
  return (scrolltext);
}


/* ----- create empty TextField widget */
Widget XmCreateTextFieldWidget (String resourcename, Widget parent)
{
  return (XtVaCreateManagedWidget (resourcename, xmTextWidgetClass, parent,
                                   NULL));
}


/* ----- create TextField widget with given width and initial string */
Widget XmCreateTextFieldWidgetW (String resourcename, Widget parent,
                                 int width, String value)
{
  return (XtVaCreateManagedWidget (resourcename, xmTextWidgetClass, parent,
                                   XmNvalue, value, XmNwidth, width, NULL));
}

/* ----- create MenuBar widget with exactly one entry */
Widget XmCreateTrivialMenuBar (Widget parent, String resourcename,
                               XmString menuname, char key)
{
  Widget menubar = XmVaCreateSimpleMenuBar (parent, "FileBrowser",
                           XmVaCASCADEBUTTON, menuname, key, NULL);
  XtVaSetValues (parent, XmNmenuBar, menubar, NULL);
  XtManageChild (menubar);
  return menubar;
}


/* ----- install callback function with type-checked resource name */
void XtAddCallbackF (Widget w, XmFuncResourceName r, 
                     void (*f) (Widget, XtPointer, XtPointer),
                     XtPointer client_data)
{
  XtAddCallback (w, r.a, f, client_data);
}


/* ----- read a value of a resource from a widget */
void XtGetIntValue (Widget w, XmStringResourceName r, int *value)
{
   Arg args[2];
   int n = 0;
   XtSetArg (args[n], r.a, value); /* store resourcename/resultpointer pair */
   XtGetValues (w, args, n);     /* retrieve value into *value */
}


void XtGetFloatValue (Widget w, XmFloatResourceName r, double *value)
{
   Arg args[2];
   int n = 0;
   XtSetArg (args[n], r.a, value); /* store resourcename/resultpointer pair */
   XtGetValues (w, args, n);     /* retrieve value into *value */
}


void XtGetStringValue (Widget w, XmStringResourceName r, String *value)
{
   Arg args[2];
   int n = 0;
   XtSetArg (args[n], r.a, value); n++; /* store resource/resultpointer pair */
   XtGetValues (w, args, n);     /* retrieve value into *value */
}

void XtGetWidgetValue (Widget w, XmStringResourceName r, Widget *value)
{
   Arg args[2];
   int n = 0;
   XtSetArg (args[n], r.a, value); n++; /* store resource/resultpointer pair */
   XtGetValues (w, args, n);     /* retrieve value into *value */
}


/* ----- change the value of a resource for a widget */
void XtSetIntValue (Widget w, XmIntResourceName r, int value)
{
   Arg args[2];
   int n = 0;
   XtSetArg (args[n], r.a, value); n++; /* store resource/resultpointer pair */
   XtSetValues (w, args, n);     /* store value into resource */
}


void XtSetFloatValue (Widget w, XmFloatResourceName r, double value)
{
   Arg args[2];
   int n = 0;
   XtSetArg (args[n], r.a, value); n++; /* store resource/resultpointer pair */
   XtSetValues (w, args, n);     /* store value into resource */
}


void XtSetStringValue (Widget w, XmStringResourceName r, String value)
{
   Arg args[2];
   int n = 0;
   XtSetArg (args[n], r.a, value); n++; /* store resource/resultpointer pair */
   XtSetValues (w, args, n);     /* store value into resource */
}


void XtSetWidgetValue (Widget w, XmWidgetResourceName r, Widget value)
{
   Arg args[2];
   int n = 0;
   XtSetArg (args[n], r.a, value); n++; /* store resource/resultpointer pair */
   XtSetValues (w, args, n);     /* store value into resource */
}



/*************** other extensions ********************************/

/* ----- fallback resources ----- */
String fallbacks[] = {
   "*fontList: fixed=NORMAL,6x10=SMALL,12x24=LARGE,9x15b=MEDIUM",
   NULL };

/* ----- convert floating point number into String of given form */
String ftoa (double x, int width, int precision)
{
   String result = malloc (width + 4);
   sprintf (result, "%*.*f", width, precision, x);
   return (result);
}


/* ----- global Initialization (protocol stamp) */
void globalInitialize (String programname)
{
  char syscall[100];
  sprintf (syscall, "../src/stamp -run %s >>protocol%s", 
           programname, programname);
  system (syscall);
}

/* ----- callback function: store selected filename from fileselector box */
void keepSelectedFile (Widget w, XtPointer client_data,
                       XtPointer callback_data)
{
  XmFileSelectionBoxCallbackStruct *cbs = 
      (XmFileSelectionBoxCallbackStruct*) callback_data;
  int result = XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, 
                                &selected_file);
  if (!result) {
    fprintf (stderr, "Cannot retrieve selected file name\n");
    exit (1);  /* some error occured, leave the whole program */
  }
  XtUnmanageChild (w);
}                       


/* ----- print error message along with 2x2 matrix in given format */
void matrixErrorMessage (String message, double matrixcoefficients[4],
                         int width, int precision)
{
  printf ("%s\n%*.*f  %*.*f\n%*.*f  %*.*f\n", message,
          width, precision, matrixcoefficients[0],
          width, precision, matrixcoefficients[1],
          width, precision, matrixcoefficients[2],
          width, precision, matrixcoefficients[3]);
}


/* ----- read file and return all its contents in one newly allocated string */
String readWholeFile (String filename)
{
  FILE  *fp = fopen (filename, "r");
  long   length;
  String result, r;
  if (fp == NULL)
    return (strdup (""));  /* couldn't open file: return new empty string */
  fseek (fp, 0, 2);     /* jump to end */
  length = ftell (fp);  /* byte offset at end is length of file */
  fseek (fp, 0, 0);     /* jump back to beginning */
  result = malloc (length);
  if (result == NULL)   /* couldn't alloc enough mem: ret. new empty string */
    return (strdup (""));
  r = result;  /* r iterates over final bytes of result */
  while (!feof (fp)) {  /* copy whole file byte-wise */
    *r = fgetc (fp);
    r++;
  }
  r--;     /* the last byte was -1 */
  *r = 0;  /* append 0 terminator */
  return (result);  
}


/* ----- retrieve filename stored by keepSelectedFile() */
String selectedFile ()
{
  return (selected_file);
}

